# from data.dataAnalysis import show_distribution_graph
#
# from model.modelSelection import selection_model
#
#
# def build(settings_csv, settings_variables, settings_models):
#     #DataProcessing
#
#     csvdata = create_dataset(settings_csv['pathcsv'], settings_csv['delimiter'], settings_csv['subset_columns'])
#
#     csvdata = create_data_variables(csvdata, settings_csv['data'])
#
#     #csvdata.drop(csvdata[(csvdata[settings_csv['target']] > 17000)].index, inplace=True) #droppate 12 righe
#     #csvdata.drop(csvdata[(csvdata[settings_csv['target']] < 3000)].index, inplace=True) #droppate 3 righe
#
#     df = csvdata
#
#     settings_variables['media_numeric'] = settings_variables['medias']
#
#     # Aggregate variables in Numerical and Categorical
#     settings_variables['numerical'] = settings_variables['media_numeric'] + settings_variables['general_numeric']
#     settings_variables['categorical'] = settings_variables['seasonal_categorical'] + settings_variables['general_categorical']
#
#     # Sum everything up in one array
#     settings_variables['all_features'] = settings_variables['categorical'] + settings_variables['numerical']
#
#     #correlation_pairplots(settings_variables['medias'], df, settings_csv['target'])
#
#     #iterative_pair_plotting(df, 5)
#
#     #print(df.corr())
#
#     show_distribution_graph(df, settings_csv['data'], settings_csv['target'], settings_variables['medias'])
#
#     # Define X as all variables except the output variable - Y is the output variable
#     X = df[settings_variables['all_features']]
#     y = df[settings_csv['target']]
#
#
#     selection_model(df, settings_variables['medias'], settings_variables['organic'], X, y, settings_models)



import pandas as pd
import numpy as np
from sklearn import metrics
import plotly.graph_objects as go


def show_nrmse(y_actual, y_pred, verbose = False):
    # normalized root mean square error
    value = round(np.sqrt(metrics.mean_squared_error(y_actual, y_pred)) / np.mean(y_actual), 3)
    passed = "✔️" if value < 0.15 else "❌"
    if verbose:
        return value, passed
    else:
        return value

def show_mape(y_actual, y_pred, verbose = False):
    # mean absolute percentage error
    value = round(metrics.mean_absolute_error(y_actual, y_pred)/np.mean(y_actual),3)
    passed = "✔️" if value < 0.15 else "❌"
    if verbose:
        return value, passed
    else:
        return value

def show_rsquared(y_actual, y_pred, verbose = False):
    # r squared
    value = round(metrics.r2_score(y_actual, y_pred), 3)
    passed = "✔️" if value > 0.8 else "❌"
    if verbose:
        return value, passed
    else:
        return value

def show_coefficients(features, model, name_model, graph = True):
    # Given model = LinearRegression() model already executed

    # Create an array of the variables you want to check coeffs
    # features = ['g_display_cost', 'g_shopping_cost', 'g_video_cost', 'g_search_brand_cost', 'g_search_no_brand_cost',
    #             'fb_cost', 'pinterest_cost', 'b_audience_cost', 'b_search_cost', 'avg_price',
    #             'solostove_organic_traffic', 'solostove_paid_traffic', 'trend_smokeless_fire_pit']

    coeffs = model.coef_
    roas = pd.DataFrame(data=coeffs, index=features, columns=['contribution'])
    title_graph = name_model + " Model Coefficients graph"

    if graph == True:

        fig = go.Figure(
            data=[go.Bar(x=roas.index, y=roas['contribution'])],
            layout=go.Layout(
                title=go.layout.Title(text=title_graph)
            )
        )

        fig.show()
        return coeffs
    else:
        print(coeffs)
        return coeffs



import plotly.express as px
import plotly.graph_objects as go

# We can then compare the Sales distribution with media spent distribution to find correlations
# Define an array with all the media spent names
#media_spent = ['g_discovery_cost', 'g_display_cost', 'g_video_cost', 'g_search_brand_cost',
#'g_search_no_brand_cost', 'b_audience_cost', 'b_search_cost', 'pinterest_cost', 'fb_cost']
def show_target_vs_media_spent_graph(df, name_data_column, name_target_column, medias_spent):
    fig = px.line(df, x=name_data_column, y=name_target_column)
    fig.show()

    fig = px.line(df, x=name_data_column, y=medias_spent)
    fig.show()

# We may want to compare Predicted vs. Actual sales
def show_prediction_vs_actual_graph(df, name_data_column, name_target_column, name_prediction_column):
    fig = px.line(df, x=name_data_column, y=[name_target_column, name_prediction_column])
    fig.show()

# We may want to compare Predicted vs. Actual sales with error
def show_prediction_vs_actual_graph_with_error(df, target_column, prediction_column, name_data_column, name_target_column, name_model = ''):

    y_error = (prediction_column - target_column) / target_column * 100

    title_accuracy_graph = name_model + ' Model: Actual vs Predicted'

    accuracy_chart = go.Figure(
        layout=dict(height=700, width=1000, xaxis_title=name_data_column, yaxis_title=name_target_column,  title=title_accuracy_graph, legend_title='Legend'))
    accuracy_chart.add_trace(go.Scatter(name="Actual", x=df[name_data_column], y=target_column))
    accuracy_chart.add_trace(go.Scatter(name="Predicted", x=df[name_data_column], y=prediction_column))
    accuracy_chart.show()

    title_error_graph = name_model + ' Model Error of Actual vs Predicted'

    error_chart = go.Figure(
        layout=dict(xaxis_title=name_data_column, yaxis_title=name_target_column + ' error', title=title_error_graph, legend_title='Legend'))
    error_chart.add_trace(go.Scatter(name="Error", x=df[name_data_column], y=y_error))
    error_chart.show()


# from src.cassandra.data import show_prediction_vs_actual_graph_with_error
# from src.cassandra.helper import helper_nevergrad
# from model.deepLearning import deepLearning
# from model.linear import linear
# from model.logLinear import logLinear
# from src.cassandra.model.modelEvaluation.evaluation import show_coefficients
# from model.ridge import ridge
#
#
# def selection_model(df, medias, organic, X, y, settings_models):
#     # funziona
#     if 'linear' in settings_models['model']:
#         result_linear, model_linear = linear(df, medias, organic, X, y, 'Linear', settings_models['linear_metric'])
#         show_prediction_vs_actual_graph_with_error(result_linear, y, result_linear['prediction'], 'ordine_data', 'revenue', 'Linear')
#         if settings_models['linear_coeffs'] == True:
#             show_coefficients(X.columns, model_linear, 'Linear', graph=True)
#
#     # chiedere feadback se funziona
#     elif 'logLinear' in settings_models['model']:
#         result_log_linear, model_log_linear = logLinear(df, medias, organic, X, y, 'Log Linear', settings_models['log_linear_metric'])
#         show_prediction_vs_actual_graph_with_error(result_log_linear, np.log(y), result_log_linear['prediction'], 'ordine_data',
#                                          'revenue', 'Log Linear')
#         if settings_models['log_linear_coeffs'] == True:
#             show_coefficients(X.columns, model_log_linear, 'Log Linear', graph=True)
#
#     # funziona
#     elif 'ridge' in settings_models['model']:
#         result_ridge, model_ridge = ridge(df, medias, organic, X, y, 'Ridge', settings_models['ridge_metric'])
#         show_prediction_vs_actual_graph_with_error(result_ridge, y, result_ridge['prediction'], 'ordine_data', 'revenue', 'Ridge',)
#         if settings_models['ridge_coeffs'] == True:
#             show_coefficients(X.columns, model_ridge, 'Ridge',graph=True)
#
#     # da capire e chiedere feadback se funziona
#     elif 'deepLearning' in settings_models['model']:
#         result_deep_learning, model_deep_learning = deepLearning(df, medias, organic, X, y, 'Deep Learning',
#                                                                  settings_models['deep_learning_metric'])
#         show_prediction_vs_actual_graph_with_error(result_deep_learning, y, result_deep_learning['prediction'], 'ordine_data',
#                                          'revenue', 'Deep Learning')
#         if settings_models['deep_learning_coeffs']:
#             show_coefficients(X.columns, model_deep_learning, 'Deep Learning', graph=True)
#
#     elif 'nevergrad' in settings_models['model']:
#         helper_nevergrad()
#
#     elif 'all' in settings_models['model']:
#     #LINEAR
#         result_linear, model_linear = linear(df, medias, organic, X, y, 'Linear', settings_models['linear_metric'])
#         show_prediction_vs_actual_graph_with_error(result_linear, y, result_linear['prediction'], 'ordine_data', 'revenue',
#                                          'Linear')
#         if settings_models['linear_coeffs'] == True:
#             show_coefficients(X.columns, model_linear, 'Linear', graph=True)
#
#     #LOG LINEAR
#         result_log_linear, model_log_linear = logLinear(df, medias, organic, X, y, 'Log Linear',
#                                                         settings_models['log_linear_metric'])
#         show_prediction_vs_actual_graph_with_error(result_log_linear, np.log(y), result_log_linear['prediction'],
#                                          'ordine_data',
#                                          'revenue', 'Log Linear')
#         if settings_models['log_linear_coeffs'] == True:
#             show_coefficients(X.columns, model_log_linear, 'Log Linear', graph=True)
#     #RIDGE
#         result_ridge, model_ridge = ridge(df, medias, organic, X, y, 'Ridge', settings_models['ridge_metric'])
#         show_prediction_vs_actual_graph_with_error(result_ridge, y, result_ridge['prediction'], 'ordine_data', 'revenue',
#                                          'Ridge', )
#         if settings_models['ridge_coeffs'] == True:
#             show_coefficients(X.columns, model_ridge, 'Ridge', graph=True)
#
#     #DEEP LEARNING
#         result_deep_learning, model_deep_learning = deepLearning(df, medias, organic, X, y, 'Deep Learning',
#                                                                  settings_models['deep_learning_metric'])
#         show_prediction_vs_actual_graph_with_error(result_deep_learning, y, result_deep_learning['prediction'], 'ordine_data',
#                                          'revenue', 'Deep Learning')
#         if settings_models['deep_learning_coeffs']:
#             show_coefficients(X.columns, model_deep_learning, 'Deep Learning', graph=True)
#
#     #NEVERGRAD
#         helper_nevergrad()



from sklearn.linear_model import LinearRegression
from sklearn.model_selection import train_test_split
from cassandra.data.trasformations.trasformations import create_model
from cassandra.model.modelEvaluation.evaluation import show_nrmse, show_mape


def linear(df, medias, organic, X, y, target, name_model, metric = ['accuracy', 'nrmse', 'mape'], return_metric = False):
    metrics_values = {}

    X_train, X_test, y_train, y_test = train_test_split(X, y)

    model = create_model(medias, organic, LinearRegression())

    model.fit(X_train, y_train)

    # Ask the model to predict on X_test without having Y_test
    # This will give you exact predicted values

    # Score returns the accuracy of the above prediction or R^2
    if 'accuracy' in metric:
        accuracy = model.score(X_test, y_test)
        if return_metric:
            metrics_values[name_model + '_accuracy'] = accuracy
        print(name_model, 'Accuracy: ', accuracy)

    # We can use our NRMSE and MAPE functions as well

    # Create new DF not to edit the original one
    result = df

    # Create a new column with predicted values
    result['prediction'] = model.predict(result)

    # Get the NRMSE & MAPE values
    if 'nrmse' in metric:
        nrmse_val = show_nrmse(result[target], result['prediction'])
        if return_metric:
            metrics_values[name_model + '_nrmse'] = nrmse_val
        print(name_model, 'NRMSE: ', nrmse_val)

    if 'mape' in metric:
        mape_val = show_mape(result[target], result['prediction'])
        if return_metric:
            metrics_values[name_model + '_mape'] = mape_val
        print(name_model, 'MAPE: ', mape_val)

    if metrics_values:
        return result, model, metrics_values
    else:
        return result, model


import numpy as np
from sklearn.base import BaseEstimator, TransformerMixin
from sklearn.utils.validation import check_is_fitted, check_array
from scipy.signal import convolve2d
from sklearn.compose import ColumnTransformer
from sklearn.pipeline import Pipeline
from sklearn.impute import SimpleImputer
import statsmodels.tsa.api as tsa

# Define the Adstock Class as a custom Transformer
class ExponentialCarryover(BaseEstimator, TransformerMixin):
    def __init__(self, strength=0.5, length=1):
        self.strength = strength
        self.length = length

    def fit(self, X, y=None):
        X = check_array(X)
        self._check_n_features(X, reset=True)
        self.sliding_window_ = (
                self.strength ** np.arange(self.length + 1)
        ).reshape(-1, 1)

        return self

    def transform(self, X: np.ndarray):
        check_is_fitted(self)
        X = check_array(X)
        self._check_n_features(X, reset=False)
        convolution = convolve2d(X, self.sliding_window_)

        if self.length > 0:
            convolution = convolution[: -self.length]

        return convolution


# Define the Saturation Class as a custom Transformer
class ExponentialSaturation:
    def __init__(self, a=1.):
        self.a = a

    def transform(self, X):
        return 1 - np.exp(-self.a * X)


class ExponentialSaturation(BaseEstimator, TransformerMixin):
    def __init__(self, a=1.):
        self.a = a

    def fit(self, X, y=None):
        X = check_array(X)
        self._check_n_features(X, reset=True)  # from BaseEstimator

        return self

    def transform(self, X):
        check_is_fitted(self)
        X = check_array(X)
        self._check_n_features(X, reset=False)  # from BaseEstimator

        return 1 - np.exp(-self.a * X)

def trasformation(media, organic):
    trasf = []
    for x in media:
         trasf.append(
             (x, Pipeline([
                               ('carryover', ExponentialCarryover()),
                               ('saturation', ExponentialSaturation())
             ]), [x]),
         )
    for x in organic:
        trasf.append(
            (x, Pipeline([
                           ('inputer', SimpleImputer())
         ]), [x]),
        )
    return trasf

def create_model(medias, organic, model):
    trasf = ColumnTransformer(
        trasformation(medias, organic)
    )

    steps = [
        ('trasformation', trasf),
        ('regression', model)
    ]
    model = Pipeline(steps)

    return model

# Function to return Adstocked variables
def adstock(x, theta):
    return tsa.filters.recursive_filter(x, theta)

# Function to return Saturated variables
def saturation(x, beta):
    return x ** beta



#!/usr/bin/env python
# coding: utf-8

# In[ ]:


from cassandra.data.dataProcessing.clean_and_format import create_data_columns
from cassandra.data.dataAnalysis.correlation import show_saturation_curves, show_relationship_between_variables
from cassandra.data.dataAnalysis.plot import show_target_vs_media_spent_graph, show_prediction_vs_actual_graph
from cassandra.data.dataProcessing.clean_and_format import create_dataset
from cassandra.model.deepLearning import deepLearning
from cassandra.model.linear import linear
from cassandra.model.logLinear import logLinear
from cassandra.model.ridge import ridge
from cassandra.data.featureSelection.ols import ols
from cassandra.data.trasformations.trasformations import saturation, adstock
from cassandra.model.modelEvaluation.evaluation import show_coefficients
from cassandra.model.modelEvaluation.evaluation import show_nrmse, show_mape
import pandas as pd
import nevergrad as ng
from sklearn.linear_model import LinearRegression
from sklearn.model_selection import train_test_split

# In[256]:


csv_data = create_dataset('D:/workspace_cassandra/mmm/spedire/dataset-spedire-07-01-22.csv',
                          subset_columns=['ordine_data', 'revenue', 'transazioni', 'google_search_spent',
                                          'google_performance_max_spent', 'google_display_spent',
                                          'fb_retargeting_spent', 'fb_prospecting_spent', 'bing_spent', 'bing_organico',
                                          'google_organico', 'referral', 'email_sessioni', 'cambio_eur_dollaro',
                                          'ios_14', 'Offerte'])
csv_data = create_data_columns(csv_data, 'ordine_data')

csv_data.drop(csv_data[(csv_data['revenue'] > 17000)].index, inplace=True)  # droppate 12 righe
csv_data.drop(csv_data[(csv_data['revenue'] < 3000)].index, inplace=True)  # droppate 3 righe

df = csv_data
# pd.set_option("display.max_rows", None)
# pd.set_option("display.max_columns", None)
pd.set_option("display.max_rows", 10)
pd.set_option("display.max_columns", 10)
df

# In[257]:


medias = [col for col in df.columns if 'spent' in col]
# medias.remove("google_discovery_spent")

show_saturation_curves(medias, df, 'revenue')

# In[258]:


# show_relationship_between_variables(df, 5)


# In[259]:


pd.set_option("display.max_rows", None)
pd.set_option("display.max_columns", None)
df.corr()

# In[260]:


show_target_vs_media_spent_graph(df, 'ordine_data', 'revenue', medias)

# In[261]:


organic = ['bing_organico', 'google_organico', 'referral', 'email_sessioni', 'cambio_eur_dollaro', 'ios_14', 'Offerte',
           'mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun', 'month', 'year']

seasonal_categorical = ['month', 'year', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun']

media_numeric = medias

general_categorical = ['ios_14', 'Offerte']

general_numeric = ['bing_organico', 'google_organico', 'referral', 'email_sessioni', 'cambio_eur_dollaro']

# Aggregate variables in Numerical and Categorical
numerical = media_numeric + general_numeric
categorical = seasonal_categorical + general_categorical

# Sum everything up in one array
all_features = categorical + numerical

df.fillna(0, inplace=True)

X = df[all_features]
y = df['revenue']

result_ols, model_ols = ols(X, y)

# In[262]:


result_linear, model_linear = linear(df, medias, organic, X, y, 'revenue', 'Linear', ['accuracy', 'nrmse', 'mape'])

# In[263]:


# result_loglinear, model_loglinear = logLinear(df, medias, organic, X, y, 'revenue', 'LogLinear', ['accuracy', 'nrmse', 'mape'])


# In[264]:


# result_ridge, model_ridge = ridge(df, medias, organic, X, y, 'revenue', 'Ridge', metric = ['accuracy', 'nrmse', 'mape'])


# In[265]:


# result_deep, model_deep = deepLearning(df, X, y, 'revenue', 'DeepLearning', ['accuracy', 'nrmse', 'mape'])


# In[266]:


new_X = {}


def build_model(google_search_spent_theta, google_search_spent_beta,
                google_performance_max_spent_theta, google_performance_max_spent_beta,
                google_display_spent_theta, google_display_spent_beta,
                fb_retargeting_spent_theta, fb_retargeting_spent_beta,
                fb_prospecting_spent_theta, fb_prospecting_spent_beta,
                bing_spent_theta, bing_spent_beta,
                bing_organico_theta, bing_organico_beta,
                google_organico_theta, google_organico_beta,
                referral_theta, referral_beta,
                email_sessioni_theta, email_sessioni_beta,
                cambio_eur_dollaro_theta, cambio_eur_dollaro_beta,
                month_theta, month_beta,
                year_theta, year_beta,
                mon_theta, mon_beta,
                tue_theta, tue_beta,
                wed_theta, wed_beta,
                thu_theta, thu_beta,
                fri_theta, fri_beta,
                sat_theta, sat_beta,
                sun_theta, sun_beta,
                ios_14_theta, ios_14_beta,
                Offerte_theta, Offerte_beta):
    # ' Transform all media variables and set them in the new Dictionary
    # ' Adstock first and Saturation second
    new_X["google_search_spent"] = saturation(adstock(df["google_search_spent"], google_search_spent_theta),
                                              google_search_spent_beta)
    new_X["google_performance_max_spent"] = saturation(
        adstock(df["google_performance_max_spent"], google_performance_max_spent_theta),
        google_performance_max_spent_beta)
    new_X["google_display_spent"] = saturation(adstock(df["google_display_spent"], google_display_spent_theta),
                                               google_display_spent_beta)
    new_X["fb_retargeting_spent"] = saturation(adstock(df["fb_retargeting_spent"], fb_retargeting_spent_theta),
                                               fb_retargeting_spent_beta)
    new_X["fb_prospecting_spent"] = saturation(adstock(df["fb_prospecting_spent"], fb_prospecting_spent_theta),
                                               fb_prospecting_spent_beta)
    new_X["bing_spent"] = saturation(adstock(df["bing_spent"], bing_spent_theta), bing_spent_beta)
    new_X["bing_organico"] = saturation(adstock(df["bing_organico"], bing_organico_theta), bing_organico_beta)
    new_X["google_organico"] = saturation(adstock(df["google_organico"], google_organico_theta), google_organico_beta)
    new_X["referral"] = saturation(adstock(df["referral"], referral_theta), referral_beta)
    new_X["email_sessioni"] = saturation(adstock(df["email_sessioni"], email_sessioni_theta), email_sessioni_beta)
    new_X["cambio_eur_dollaro"] = saturation(adstock(df["cambio_eur_dollaro"], cambio_eur_dollaro_theta),
                                             cambio_eur_dollaro_beta)
    new_X["month"] = saturation(adstock(df["month"], month_theta), month_beta)
    new_X["year"] = saturation(adstock(df["year"], year_theta), year_beta)
    new_X["mon"] = saturation(adstock(df["mon"], mon_theta), mon_beta)
    new_X["tue"] = saturation(adstock(df["tue"], tue_theta), tue_beta)
    new_X["wed"] = saturation(adstock(df["wed"], wed_theta), wed_beta)
    new_X["thu"] = saturation(adstock(df["thu"], thu_theta), thu_beta)
    new_X["fri"] = saturation(adstock(df["fri"], fri_theta), fri_beta)
    new_X["sat"] = saturation(adstock(df["sat"], sat_theta), sat_beta)
    new_X["sun"] = saturation(adstock(df["sun"], sun_theta), sun_beta)
    new_X["ios_14"] = saturation(adstock(df["ios_14"], ios_14_theta), ios_14_beta)
    new_X["Offerte"] = saturation(adstock(df["Offerte"], Offerte_theta), Offerte_beta)

    # ' Cast Dictionary to DataFrame and append the output column
    new_df = pd.DataFrame.from_dict(new_X)
    new_df = new_df.join(df['revenue'])
    # print(new_df)

    # ' Train test split data
    X = new_df[all_features]
    y = new_df['revenue']

    X_train, X_test, y_train, y_test = train_test_split(X, y)

    model = LinearRegression()
    model.fit(X_train, y_train)

    result = df
    # print(result)
    result['prediction'] = model.predict(X)

    nrmse_val = show_nrmse(result['revenue'], result['prediction'])
    mape_val = show_mape(result['revenue'], result['prediction'])
    accuracy = model.score(X_test, y_test)

    # print(nrmse_val, mape_val, accuracy)
    return mape_val


# In[267]:


# ' Define the list of hyperparameters to optimize
# ' List must be the same as the ones in the function's definition, same order recommended too
instrum = ng.p.Instrumentation(
    google_search_spent_theta=ng.p.Scalar(lower=0, upper=1),
    google_search_spent_beta=ng.p.Scalar(lower=0, upper=1),

    google_performance_max_spent_theta=ng.p.Scalar(lower=0, upper=1),
    google_performance_max_spent_beta=ng.p.Scalar(lower=0, upper=1),

    google_display_spent_theta=ng.p.Scalar(lower=0, upper=1),
    google_display_spent_beta=ng.p.Scalar(lower=0, upper=1),

    fb_retargeting_spent_theta=ng.p.Scalar(lower=0, upper=1),
    fb_retargeting_spent_beta=ng.p.Scalar(lower=0, upper=1),

    fb_prospecting_spent_theta=ng.p.Scalar(lower=0, upper=1),
    fb_prospecting_spent_beta=ng.p.Scalar(lower=0, upper=1),

    bing_spent_theta=ng.p.Scalar(lower=0, upper=1),
    bing_spent_beta=ng.p.Scalar(lower=0, upper=1),

    bing_organico_theta=ng.p.Scalar(lower=0, upper=1),
    bing_organico_beta=ng.p.Scalar(lower=0, upper=1),

    google_organico_theta=ng.p.Scalar(lower=0, upper=1),
    google_organico_beta=ng.p.Scalar(lower=0, upper=1),

    referral_theta=ng.p.Scalar(lower=0, upper=1),
    referral_beta=ng.p.Scalar(lower=0, upper=1),

    email_sessioni_theta=ng.p.Scalar(lower=0, upper=1),
    email_sessioni_beta=ng.p.Scalar(lower=0, upper=1),

    cambio_eur_dollaro_theta=ng.p.Scalar(lower=0, upper=1),
    cambio_eur_dollaro_beta=ng.p.Scalar(lower=0, upper=1),

    month_theta=ng.p.Scalar(lower=0, upper=1),
    month_beta=ng.p.Scalar(lower=0, upper=1),

    year_theta=ng.p.Scalar(lower=0, upper=1),
    year_beta=ng.p.Scalar(lower=0, upper=1),

    mon_theta=ng.p.Scalar(lower=0, upper=1),
    mon_beta=ng.p.Scalar(lower=0, upper=1),

    tue_theta=ng.p.Scalar(lower=0, upper=1),
    tue_beta=ng.p.Scalar(lower=0, upper=1),

    wed_theta=ng.p.Scalar(lower=0, upper=1),
    wed_beta=ng.p.Scalar(lower=0, upper=1),

    thu_theta=ng.p.Scalar(lower=0, upper=1),
    thu_beta=ng.p.Scalar(lower=0, upper=1),

    fri_theta=ng.p.Scalar(lower=0, upper=1),
    fri_beta=ng.p.Scalar(lower=0, upper=1),

    sat_theta=ng.p.Scalar(lower=0, upper=1),
    sat_beta=ng.p.Scalar(lower=0, upper=1),

    sun_theta=ng.p.Scalar(lower=0, upper=1),
    sun_beta=ng.p.Scalar(lower=0, upper=1),

    ios_14_theta=ng.p.Scalar(lower=0, upper=1),
    ios_14_beta=ng.p.Scalar(lower=0, upper=1),

    Offerte_theta=ng.p.Scalar(lower=0, upper=1),
    Offerte_beta=ng.p.Scalar(lower=0, upper=1)
)

# print(instrum)
# ' Define an Optimizer (use NGOpt as default) and set budget as number of trials (recommended 2500+)
optimizer = ng.optimizers.NGOpt(parametrization=instrum, budget=2500)
# print(optimizer)

# ' Pass the function to minimize
# ' Nevergrad will automatically map Hyperparams
recommendation = optimizer.minimize(build_model)

# ' Results of the optimization are inside the following variable
recommendation.value

# build_model(**recommendation.value[1])
# ' Use the following code to directly use the optimized hyperparams to build you model


# In[268]:


new_X = {}


def build_model(google_search_spent_theta, google_search_spent_beta,
                google_performance_max_spent_theta, google_performance_max_spent_beta,
                google_display_spent_theta, google_display_spent_beta,
                fb_retargeting_spent_theta, fb_retargeting_spent_beta,
                fb_prospecting_spent_theta, fb_prospecting_spent_beta,
                bing_spent_theta, bing_spent_beta,
                bing_organico_theta, bing_organico_beta,
                google_organico_theta, google_organico_beta,
                referral_theta, referral_beta,
                email_sessioni_theta, email_sessioni_beta,
                cambio_eur_dollaro_theta, cambio_eur_dollaro_beta,
                month_theta, month_beta,
                year_theta, year_beta,
                mon_theta, mon_beta,
                tue_theta, tue_beta,
                wed_theta, wed_beta,
                thu_theta, thu_beta,
                fri_theta, fri_beta,
                sat_theta, sat_beta,
                sun_theta, sun_beta,
                ios_14_theta, ios_14_beta,
                Offerte_theta, Offerte_beta):
    # ' Transform all media variables and set them in the new Dictionary
    # ' Adstock first and Saturation second
    new_X["google_search_spent"] = saturation(adstock(df["google_search_spent"], google_search_spent_theta),
                                              google_search_spent_beta)
    new_X["google_performance_max_spent"] = saturation(
        adstock(df["google_performance_max_spent"], google_performance_max_spent_theta),
        google_performance_max_spent_beta)
    new_X["google_display_spent"] = saturation(adstock(df["google_display_spent"], google_display_spent_theta),
                                               google_display_spent_beta)
    new_X["fb_retargeting_spent"] = saturation(adstock(df["fb_retargeting_spent"], fb_retargeting_spent_theta),
                                               fb_retargeting_spent_beta)
    new_X["fb_prospecting_spent"] = saturation(adstock(df["fb_prospecting_spent"], fb_prospecting_spent_theta),
                                               fb_prospecting_spent_beta)
    new_X["bing_spent"] = saturation(adstock(df["bing_spent"], bing_spent_theta), bing_spent_beta)
    new_X["bing_organico"] = saturation(adstock(df["bing_organico"], bing_organico_theta), bing_organico_beta)
    new_X["google_organico"] = saturation(adstock(df["google_organico"], google_organico_theta), google_organico_beta)
    new_X["referral"] = saturation(adstock(df["referral"], referral_theta), referral_beta)
    new_X["email_sessioni"] = saturation(adstock(df["email_sessioni"], email_sessioni_theta), email_sessioni_beta)
    new_X["cambio_eur_dollaro"] = saturation(adstock(df["cambio_eur_dollaro"], cambio_eur_dollaro_theta),
                                             cambio_eur_dollaro_beta)
    new_X["month"] = saturation(adstock(df["month"], month_theta), month_beta)
    new_X["year"] = saturation(adstock(df["year"], year_theta), year_beta)
    new_X["mon"] = saturation(adstock(df["mon"], mon_theta), mon_beta)
    new_X["tue"] = saturation(adstock(df["tue"], tue_theta), tue_beta)
    new_X["wed"] = saturation(adstock(df["wed"], wed_theta), wed_beta)
    new_X["thu"] = saturation(adstock(df["thu"], thu_theta), thu_beta)
    new_X["fri"] = saturation(adstock(df["fri"], fri_theta), fri_beta)
    new_X["sat"] = saturation(adstock(df["sat"], sat_theta), sat_beta)
    new_X["sun"] = saturation(adstock(df["sun"], sun_theta), sun_beta)
    new_X["ios_14"] = saturation(adstock(df["ios_14"], ios_14_theta), ios_14_beta)
    new_X["Offerte"] = saturation(adstock(df["Offerte"], Offerte_theta), Offerte_beta)

    # ' Cast Dictionary to DataFrame and append the output column
    new_df = pd.DataFrame.from_dict(new_X)
    new_df = new_df.join(df['revenue'])

    # ' Train test split data
    X = new_df[all_features]
    y = new_df['revenue']

    X_train, X_test, y_train, y_test = train_test_split(X, y)

    model = LinearRegression()
    model.fit(X_train, y_train)

    result = df
    result['prediction'] = model.predict(X)

    nrmse_val = show_nrmse(result['revenue'], result['prediction'])
    mape_val = show_mape(result['revenue'], result['prediction'])
    accuracy = model.score(X_test, y_test)

    print(nrmse_val, mape_val, accuracy)
    return mape_val, model, result


# In[269]:


mape_val, model, result = build_model(**recommendation.value[1])
print(mape_val)

# In[271]:


show_prediction_vs_actual_graph(result, 'ordine_data', 'revenue', 'prediction')

# In[278]:


show_coefficients(all_features, model, 'Nevergrad')

# In[ ]:







import statsmodels.api as sm


def ols(X, y, constant = True):

    if constant == True:
        X = sm.add_constant(X)

    #model = create_model(medias, organic, sm.OLS(y, X))
    model = sm.OLS(y, X)

    result = model.fit()
    print(result.summary())

    return result, model